 /************************************************************************
 *   MODULE  : SWAP.C
 *
 *   PURPOSE : contains functions which determine the machine byte order
 *             and swap bytes to maintain portability
 *
 *   AUTHOR  : Kevin Russo, SAIC (original)
 *
 *   Change     Date          Reason
 *   ------     ----          ------
 *   Original   12 Apr 93     n/a
 *
 ************************************************************************/

/****************
 * header files *
 ****************/

#include "los.h"

/************************************************************************
 *   FUNCTION     : big_endian_machine
 *
 *   PURPOSE      : determines the machine byte order
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : TRUE  - machine uses big endian byte order
 *                  FALSE - does not use big endian byte order
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
BOOLEAN big_endian_machine (void)
#else
BOOLEAN big_endian_machine ()
#endif
{
    char   *end_char;           /* char pointer to the bif endian check */
    short  end_check;           /* value to perform endian check upon   */
    short  big_endian;          /* final result of the test             */

    end_check  = (short)ONE;

    end_char   = (char *)&end_check;

    big_endian = ONE - (short)(*end_char);

    if (big_endian)
       return TRUE;

   return FALSE;
}

/************************************************************************
 *   FUNCTION     : swap_short
 *
 *   PURPOSE      : reverses the byte order for a short (2 byte) number
 *
 *   INPUTS       : num1 - number to be swapped
 *
 *   RETURN VALUE : num2 - num1 swapped
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
short swap_short (short num1)
#else
short swap_short (num1)
short num1;
#endif
{
   short mask1 = 0x00ff, mask2 = 0xff00;

   short temp[2], num2;

   temp[0] = (num1 & mask1) << 8;

   temp[1] = ((num1 & mask2) >> 8) & mask1;

   num2  = temp[0] | temp[1];

   return num2;
}

/************************************************************************
 *   FUNCTION     : swap_long
 *
 *   PURPOSE      : reverses the byte order for a long (4 byte) number
 *
 *   INPUTS       : num1 - number to be swapped
 *
 *   RETURN VALUE : num2 - num1 swapped
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
long swap_long (long num1)
#else
long swap_long (num1)
long num1;
#endif
{
   long mask1 = 0x000000ffL, mask2 = 0x0000ff00L;
   long mask3 = 0x00ff0000L, mask4 = 0xff000000L;

   long temp[4], num2;

   temp[0] = (num1 & mask1) << 24;
   temp[1] = (num1 & mask2) << 8;

   temp[2] = ((num1 & mask3) >>  8) & mask2;
   temp[3] = ((num1 & mask4) >> 24) & mask1;

   num2  = temp[0] | temp[1] | temp[2] | temp[3];

   return num2;
}

/************************************************************************
 *   FUNCTION     : swap_float
 *
 *   PURPOSE      : reverses the byte order for a float (4 byte) number
 *
 *   INPUTS       : num1 - number to be swapped
 *
 *   RETURN VALUE : num2 - num1 swapped
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
float swap_float (float num1)
#else
float swap_float (num1)
float num1;
#endif
{
   char *num_loc, temp_ch;
   float num2;

   num2     = num1;
   num_loc  = (char *)&num2;
 
   temp_ch    = num_loc[0];
   num_loc[0] = num_loc[3];
   num_loc[3] = temp_ch;
   
   temp_ch    = num_loc[1];  
   num_loc[1] = num_loc[2];
   num_loc[2] = temp_ch;

   return num2;
}

/************************************************************************
 *   FUNCTION     : swap_double
 *
 *   PURPOSE      : reverses the byte order for a double (8 byte) number
 *
 *   INPUTS       : num1 - number to be swapped
 *
 *   RETURN VALUE : num2 - num1 swapped
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
#if XVT_OS == XVT_OS_MAC
short double swap_double (short double num1)
#else
double swap_double (double num1)
#endif
#else
#if XVT_OS == XVT_OS_MAC
short double swap_double (num1)
short double num1;
#else
double swap_double (num1)
double num1;
#endif
#endif
{
   char   *num_loc, temp_ch;
   short  loop;

#if XVT_OS == XVT_OS_MAC
   short double num2;
#else   
   double num2;
#endif

   num2     = num1;
   num_loc  = (char *)&num2;

   for (loop = 0; loop < 4; loop++)
   {
      temp_ch         = num_loc[loop];
      num_loc[loop]   = num_loc[7 - loop];
      num_loc[7-loop] = temp_ch;
   }

   return num2;
}
